@@ -23,10 +23,21 @@ use std::io::Read;
 pub const CODEC_METADATA_KEY: &str = "avro.codec";
 
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
+/// Supported compression codecs for Avro data
+///
+/// Avro supports multiple compression formats for data blocks.
+/// This enum represents the compression codecs available in this implementation.
 pub enum CompressionCodec {
+    /// Deflate compression (RFC 1951)
     Deflate,
+    /// Snappy compression
     Snappy,
+    /// ZStandard compression
     ZStandard,
+    /// Bzip2 compression
+    Bzip2,
+    /// Xz compression
+    Xz,
 }
 
 impl CompressionCodec {
@@ -77,6 +88,28 @@ impl CompressionCodec {
             CompressionCodec::ZStandard => Err(ArrowError::ParseError(
                 "ZStandard codec requires zstd feature".to_string(),
             )),
+            #[cfg(feature = "bzip2")]
+            CompressionCodec::Bzip2 => {
+                let mut decoder = bzip2::read::BzDecoder::new(block);
+                let mut out = Vec::new();
+                decoder.read_to_end(&mut out)?;
+                Ok(out)
+            }
+            #[cfg(not(feature = "bzip2"))]
+            CompressionCodec::Bzip2 => Err(ArrowError::ParseError(
+                "Bzip2 codec requires bzip2 feature".to_string(),
+            )),
+            #[cfg(feature = "xz")]
+            CompressionCodec::Xz => {
+                let mut decoder = xz::read::XzDecoder::new(block);
+                let mut out = Vec::new();
+                decoder.read_to_end(&mut out)?;
+                Ok(out)
+            }
+            #[cfg(not(feature = "xz"))]
+            CompressionCodec::Xz => Err(ArrowError::ParseError(
+                "XZ codec requires xz feature".to_string(),
+            )),
         }
     }
 }
