@@ -15,11 +15,13 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Maximum and minimum values for [`Decimal256`] and [`Decimal128`].
+//! Maximum and minimum values for [`Decimal256`], [`Decimal128`], [`Decimal64`] and [`Decimal32`].
 //!
 //! Also provides functions to validate if a given decimal value is within
 //! the valid range of the decimal type.
 //!
+//! [`Decimal32`]: arrow_schema::DataType::Decimal32
+//! [`Decimal64`]: arrow_schema::DataType::Decimal64
 //! [`Decimal128`]: arrow_schema::DataType::Decimal128
 //! [`Decimal256`]: arrow_schema::DataType::Decimal256
 use arrow_buffer::i256;
@@ -27,7 +29,8 @@ use arrow_schema::ArrowError;
 
 pub use arrow_schema::{
     DECIMAL128_MAX_PRECISION, DECIMAL128_MAX_SCALE, DECIMAL256_MAX_PRECISION, DECIMAL256_MAX_SCALE,
-    DECIMAL_DEFAULT_SCALE,
+    DECIMAL32_DEFAULT_SCALE, DECIMAL32_MAX_PRECISION, DECIMAL32_MAX_SCALE, DECIMAL64_DEFAULT_SCALE,
+    DECIMAL64_MAX_PRECISION, DECIMAL64_MAX_SCALE, DECIMAL_DEFAULT_SCALE,
 };
 
 /// `MAX_DECIMAL256_FOR_EACH_PRECISION[p]` holds the maximum [`i256`] value that can
@@ -899,6 +902,194 @@ pub const MIN_DECIMAL128_FOR_EACH_PRECISION: [i128; 39] = [
     -99999999999999999999999999999999999999,
 ];
 
+/// `MAX_DECIMAL64_FOR_EACH_PRECISION[p]` holds the maximum `i64` value that can
+/// be stored in [`Decimal64`] value of precision `p`.
+///
+/// # Notes
+///
+/// The first element is unused and is inserted so that we can look up using
+/// precision as the index without the need to subtract 1 first.
+///
+/// # Example
+/// ```
+/// # use arrow_data::decimal::MAX_DECIMAL64_FOR_EACH_PRECISION;
+/// assert_eq!(MAX_DECIMAL64_FOR_EACH_PRECISION[3], 999);
+/// ```
+///
+/// [`Decimal64`]: arrow_schema::DataType::Decimal64
+pub const MAX_DECIMAL64_FOR_EACH_PRECISION: [i64; 19] = [
+    0, // unused first element
+    9,
+    99,
+    999,
+    9999,
+    99999,
+    999999,
+    9999999,
+    99999999,
+    999999999,
+    9999999999,
+    99999999999,
+    999999999999,
+    9999999999999,
+    99999999999999,
+    999999999999999,
+    9999999999999999,
+    99999999999999999,
+    999999999999999999,
+];
+
+/// `MIN_DECIMAL64_FOR_EACH_PRECISION[p]` holds the minimum `i64` value that can
+/// be stored in a [`Decimal64`] value of precision `p`.
+///
+/// # Notes
+///
+/// The first element is unused and is inserted so that we can look up using
+/// precision as the index without the need to subtract 1 first.
+///
+/// # Example
+/// ```
+/// # use arrow_data::decimal::MIN_DECIMAL64_FOR_EACH_PRECISION;
+/// assert_eq!(MIN_DECIMAL64_FOR_EACH_PRECISION[3], -999);
+/// ```
+///
+/// [`Decimal64`]: arrow_schema::DataType::Decimal64
+pub const MIN_DECIMAL64_FOR_EACH_PRECISION: [i64; 19] = [
+    0, // unused first element
+    -9,
+    -99,
+    -999,
+    -9999,
+    -99999,
+    -999999,
+    -9999999,
+    -99999999,
+    -999999999,
+    -9999999999,
+    -99999999999,
+    -999999999999,
+    -9999999999999,
+    -99999999999999,
+    -999999999999999,
+    -9999999999999999,
+    -99999999999999999,
+    -999999999999999999,
+];
+
+/// `MAX_DECIMAL32_FOR_EACH_PRECISION[p]` holds the maximum `i32` value that can
+/// be stored in [`Decimal32`] value of precision `p`.
+///
+/// # Notes
+///
+/// The first element is unused and is inserted so that we can look up using
+/// precision as the index without the need to subtract 1 first.
+///
+/// # Example
+/// ```
+/// # use arrow_data::decimal::MAX_DECIMAL32_FOR_EACH_PRECISION;
+/// assert_eq!(MAX_DECIMAL32_FOR_EACH_PRECISION[3], 999);
+/// ```
+///
+/// [`Decimal32`]: arrow_schema::DataType::Decimal32
+pub const MAX_DECIMAL32_FOR_EACH_PRECISION: [i32; 10] = [
+    0, // unused first element
+    9, 99, 999, 9999, 99999, 999999, 9999999, 99999999, 999999999,
+];
+
+/// `MIN_DECIMAL32_FOR_EACH_PRECISION[p]` holds the minimum `ialue that can
+/// be stored in a [`Decimal32`] value of precision `p`.
+///
+/// # Notes
+///
+/// The first element is unused and is inserted so that we can look up using
+/// precision as the index without the need to subtract 1 first.
+///
+/// # Example
+/// ```
+/// # use arrow_data::decimal::MIN_DECIMAL32_FOR_EACH_PRECISION;
+/// assert_eq!(MIN_DECIMAL32_FOR_EACH_PRECISION[3], -999);
+/// ```
+///
+/// [`Decimal32`]: arrow_schema::DataType::Decimal32
+pub const MIN_DECIMAL32_FOR_EACH_PRECISION: [i32; 10] = [
+    0, // unused first element
+    -9, -99, -999, -9999, -99999, -999999, -9999999, -99999999, -999999999,
+];
+
+/// Validates that the specified `i32` value can be properly
+/// interpreted as a [`Decimal32`] number with precision `precision`
+///
+/// [`Decimal32`]: arrow_schema::DataType::Decimal32
+#[inline]
+pub fn validate_decimal32_precision(value: i32, precision: u8) -> Result<(), ArrowError> {
+    if precision > DECIMAL32_MAX_PRECISION {
+        return Err(ArrowError::InvalidArgumentError(format!(
+            "Max precision of a Decimal32 is {DECIMAL32_MAX_PRECISION}, but got {precision}",
+        )));
+    }
+    if value > MAX_DECIMAL32_FOR_EACH_PRECISION[precision as usize] {
+        Err(ArrowError::InvalidArgumentError(format!(
+            "{value} is too large to store in a Decimal32 of precision {precision}. Max is {}",
+            MAX_DECIMAL32_FOR_EACH_PRECISION[precision as usize]
+        )))
+    } else if value < MIN_DECIMAL32_FOR_EACH_PRECISION[precision as usize] {
+        Err(ArrowError::InvalidArgumentError(format!(
+            "{value} is too small to store in a Decimal32 of precision {precision}. Min is {}",
+            MIN_DECIMAL32_FOR_EACH_PRECISION[precision as usize]
+        )))
+    } else {
+        Ok(())
+    }
+}
+
+/// Returns true if the specified `i32` value can be properly
+/// interpreted as a [`Decimal32`] number with precision `precision`
+///
+/// [`Decimal32`]: arrow_schema::DataType::Decimal32
+#[inline]
+pub fn is_validate_decimal32_precision(value: i32, precision: u8) -> bool {
+    precision <= DECIMAL32_MAX_PRECISION
+        && value >= MIN_DECIMAL32_FOR_EACH_PRECISION[precision as usize]
+        && value <= MAX_DECIMAL32_FOR_EACH_PRECISION[precision as usize]
+}
+
+/// Validates that the specified `i64` value can be properly
+/// interpreted as a [`Decimal64`] number with precision `precision`
+///
+/// [`Decimal64`]: arrow_schema::DataType::Decimal64
+#[inline]
+pub fn validate_decimal64_precision(value: i64, precision: u8) -> Result<(), ArrowError> {
+    if precision > DECIMAL64_MAX_PRECISION {
+        return Err(ArrowError::InvalidArgumentError(format!(
+            "Max precision of a Decimal64 is {DECIMAL64_MAX_PRECISION}, but got {precision}",
+        )));
+    }
+    if value > MAX_DECIMAL64_FOR_EACH_PRECISION[precision as usize] {
+        Err(ArrowError::InvalidArgumentError(format!(
+            "{value} is too large to store in a Decimal64 of precision {precision}. Max is {}",
+            MAX_DECIMAL64_FOR_EACH_PRECISION[precision as usize]
+        )))
+    } else if value < MIN_DECIMAL64_FOR_EACH_PRECISION[precision as usize] {
+        Err(ArrowError::InvalidArgumentError(format!(
+            "{value} is too small to store in a Decimal64 of precision {precision}. Min is {}",
+            MIN_DECIMAL64_FOR_EACH_PRECISION[precision as usize]
+        )))
+    } else {
+        Ok(())
+    }
+}
+
+/// Returns true if the specified `i64` value can be properly
+/// interpreted as a [`Decimal64`] number with precision `precision`
+///
+/// [`Decimal64`]: arrow_schema::DataType::Decimal64
+#[inline]
+pub fn is_validate_decimal64_precision(value: i64, precision: u8) -> bool {
+    precision <= DECIMAL64_MAX_PRECISION
+        && value >= MIN_DECIMAL64_FOR_EACH_PRECISION[precision as usize]
+        && value <= MAX_DECIMAL64_FOR_EACH_PRECISION[precision as usize]
+}
+
 /// Validates that the specified `i128` value can be properly
 /// interpreted as a [`Decimal128`] number with precision `precision`
 ///
