@@ -133,8 +133,7 @@ pub fn from_thrift(
 
             if null_count < 0 {
                 return Err(ParquetError::General(format!(
-                    "Statistics null count is negative {}",
-                    null_count
+                    "Statistics null count is negative {null_count}",
                 )));
             }
 
@@ -157,6 +156,32 @@ pub fn from_thrift(
                 stats.max_value
             };
 
+            fn check_len(min: &Option<Vec<u8>>, max: &Option<Vec<u8>>, len: usize) -> Result<()> {
+                if let Some(min) = min {
+                    if min.len() < len {
+                        return Err(ParquetError::General(
+                            "Insufficient bytes to parse min statistic".to_string(),
+                        ));
+                    }
+                }
+                if let Some(max) = max {
+                    if max.len() < len {
+                        return Err(ParquetError::General(
+                            "Insufficient bytes to parse max statistic".to_string(),
+                        ));
+                    }
+                }
+                Ok(())
+            }
+
+            match physical_type {
+                Type::BOOLEAN => check_len(&min, &max, 1),
+                Type::INT32 | Type::FLOAT => check_len(&min, &max, 4),
+                Type::INT64 | Type::DOUBLE => check_len(&min, &max, 8),
+                Type::INT96 => check_len(&min, &max, 12),
+                _ => Ok(()),
+            }?;
+
             // Values are encoded using PLAIN encoding definition, except that
             // variable-length byte arrays do not include a length prefix.
             //
@@ -185,8 +210,6 @@ pub fn from_thrift(
                 ),
                 Type::INT96 => {
                     // INT96 statistics may not be correct, because comparison is signed
-                    // byte-wise, not actual timestamps. It is recommended to ignore
-                    // min/max statistics for INT96 columns.
                     let min = if let Some(data) = min {
                         assert_eq!(data.len(), 12);
                         Some(Int96::try_from_le_slice(&data)?)
@@ -393,36 +416,12 @@ impl Statistics {
         statistics_enum_func![self, is_min_max_backwards_compatible]
     }
 
-    /// Returns optional value of number of distinct values occurring.
-    /// When it is `None`, the value should be ignored.
-    #[deprecated(since = "53.0.0", note = "Use `distinct_count_opt` method instead")]
-    pub fn distinct_count(&self) -> Option<u64> {
-        self.distinct_count_opt()
-    }
-
     /// Returns optional value of number of distinct values occurring.
     /// When it is `None`, the value should be ignored.
     pub fn distinct_count_opt(&self) -> Option<u64> {
         statistics_enum_func![self, distinct_count]
     }
 
-    /// Returns number of null values for the column.
-    /// Note that this includes all nulls when column is part of the complex type.
-    ///
-    /// Note this API returns 0 if the null count is not available.
-    #[deprecated(since = "53.0.0", note = "Use `null_count_opt` method instead")]
-    pub fn null_count(&self) -> u64 {
-        // 0 to remain consistent behavior prior to `null_count_opt`
-        self.null_count_opt().unwrap_or(0)
-    }
-
-    /// Returns `true` if statistics collected any null values, `false` otherwise.
-    #[deprecated(since = "53.0.0", note = "Use `null_count_opt` method instead")]
-    #[allow(deprecated)]
-    pub fn has_nulls(&self) -> bool {
-        self.null_count() > 0
-    }
-
     /// Returns number of null values for the column, if known.
     /// Note that this includes all nulls when column is part of the complex type.
     ///
@@ -433,16 +432,6 @@ impl Statistics {
         statistics_enum_func![self, null_count_opt]
     }
 
-    /// Whether or not min and max values are set.
-    /// Normally both min/max values will be set to `Some(value)` or `None`.
-    #[deprecated(
-        since = "53.0.0",
-        note = "Use `min_bytes_opt` and `max_bytes_opt` methods instead"
-    )]
-    pub fn has_min_max_set(&self) -> bool {
-        statistics_enum_func![self, _internal_has_min_max_set]
-    }
-
     /// Returns `true` if the min value is set, and is an exact min value.
     pub fn min_is_exact(&self) -> bool {
         statistics_enum_func![self, min_is_exact]
@@ -458,25 +447,11 @@ impl Statistics {
         statistics_enum_func![self, min_bytes_opt]
     }
 
-    /// Returns slice of bytes that represent min value.
-    /// Panics if min value is not set.
-    #[deprecated(since = "53.0.0", note = "Use `max_bytes_opt` instead")]
-    pub fn min_bytes(&self) -> &[u8] {
-        self.min_bytes_opt().unwrap()
-    }
-
     /// Returns slice of bytes that represent max value, if max value is known.
     pub fn max_bytes_opt(&self) -> Option<&[u8]> {
         statistics_enum_func![self, max_bytes_opt]
     }
 
-    /// Returns slice of bytes that represent max value.
-    /// Panics if max value is not set.
-    #[deprecated(since = "53.0.0", note = "Use `max_bytes_opt` instead")]
-    pub fn max_bytes(&self) -> &[u8] {
-        self.max_bytes_opt().unwrap()
-    }
-
     /// Returns physical type associated with statistics.
     pub fn physical_type(&self) -> Type {
         match self {
@@ -589,29 +564,11 @@ impl<T: ParquetValueType> ValueStatistics<T> {
         }
     }
 
-    /// Returns min value of the statistics.
-    ///
-    /// Panics if min value is not set, e.g. all values are `null`.
-    /// Use `has_min_max_set` method to check that.
-    #[deprecated(since = "53.0.0", note = "Use `min_opt` instead")]
-    pub fn min(&self) -> &T {
-        self.min.as_ref().unwrap()
-    }
-
     /// Returns min value of the statistics, if known.
     pub fn min_opt(&self) -> Option<&T> {
         self.min.as_ref()
     }
 
-    /// Returns max value of the statistics.
-    ///
-    /// Panics if max value is not set, e.g. all values are `null`.
-    /// Use `has_min_max_set` method to check that.
-    #[deprecated(since = "53.0.0", note = "Use `max_opt` instead")]
-    pub fn max(&self) -> &T {
-        self.max.as_ref().unwrap()
-    }
-
     /// Returns max value of the statistics, if known.
     pub fn max_opt(&self) -> Option<&T> {
         self.max.as_ref()
@@ -622,36 +579,11 @@ impl<T: ParquetValueType> ValueStatistics<T> {
         self.min_opt().map(AsBytes::as_bytes)
     }
 
-    /// Returns min value as bytes of the statistics.
-    ///
-    /// Panics if min value is not set, use `has_min_max_set` method to check
-    /// if values are set.
-    #[deprecated(since = "53.0.0", note = "Use `min_bytes_opt` instead")]
-    pub fn min_bytes(&self) -> &[u8] {
-        self.min_bytes_opt().unwrap()
-    }
-
     /// Returns max value as bytes of the statistics, if max value is known.
     pub fn max_bytes_opt(&self) -> Option<&[u8]> {
         self.max_opt().map(AsBytes::as_bytes)
     }
 
-    /// Returns max value as bytes of the statistics.
-    ///
-    /// Panics if max value is not set, use `has_min_max_set` method to check
-    /// if values are set.
-    #[deprecated(since = "53.0.0", note = "Use `max_bytes_opt` instead")]
-    pub fn max_bytes(&self) -> &[u8] {
-        self.max_bytes_opt().unwrap()
-    }
-
-    /// Whether or not min and max values are set.
-    /// Normally both min/max values will be set to `Some(value)` or `None`.
-    #[deprecated(since = "53.0.0", note = "Use `min_opt` and `max_opt` methods instead")]
-    pub fn has_min_max_set(&self) -> bool {
-        self._internal_has_min_max_set()
-    }
-
     /// Whether or not min and max values are set.
     /// Normally both min/max values will be set to `Some(value)` or `None`.
     pub(crate) fn _internal_has_min_max_set(&self) -> bool {
@@ -673,14 +605,6 @@ impl<T: ParquetValueType> ValueStatistics<T> {
         self.distinct_count
     }
 
-    /// Returns number of null values for the column.
-    /// Note that this includes all nulls when column is part of the complex type.
-    #[deprecated(since = "53.0.0", note = "Use `null_count_opt` method instead")]
-    pub fn null_count(&self) -> u64 {
-        // 0 to remain consistent behavior prior to `null_count_opt`
-        self.null_count_opt().unwrap_or(0)
-    }
-
     /// Returns null count.
     pub fn null_count_opt(&self) -> Option<u64> {
         self.null_count
